@@ -22,8 +22,8 @@ import traceback
 import botocore.exceptions
 from oslo_log import log
 import six
-from tempest_lib import base
-from tempest_lib import exceptions
+from tempest.lib import base
+from tempest.lib import exceptions
 import testtools
 
 from ec2api.tests.functional import botocoreclient
@@ -189,12 +189,21 @@ class TesterStateHolder(object):
                 cls, *args, **kwargs)
         return cls._instance
 
+    _ec2_enabled = None
     _vpc_enabled = None
 
-    def get_vpc_enabled(self):
-        if self._vpc_enabled:
-            return self._vpc_enabled
+    def get_ec2_enabled(self):
+        if self._ec2_enabled is None:
+            self._fill_attributes()
+        return self._ec2_enabled
 
+    def get_vpc_enabled(self):
+        if self._vpc_enabled is None:
+            self._fill_attributes()
+        return self._vpc_enabled
+
+    def _fill_attributes(self):
+        self._ec2_enabled = False
         self._vpc_enabled = False
         data = self.ec2_client.describe_account_attributes()
         for item in data.get('AccountAttributes', []):
@@ -202,8 +211,21 @@ class TesterStateHolder(object):
                 for value in item['AttributeValues']:
                     if value['AttributeValue'] == 'VPC':
                         self._vpc_enabled = True
+                    if value['AttributeValue'] == 'EC2':
+                        self._ec2_enabled = True
 
-        return self._vpc_enabled
+
+def skip_without_ec2(*args, **kwargs):
+    """A decorator useful to skip tests if EC2-classic is not supported."""
+    def decorator(f):
+        @functools.wraps(f)
+        def wrapper(self, *func_args, **func_kwargs):
+            if not TesterStateHolder().get_ec2_enabled():
+                msg = "Skipped because EC2-classic is not enabled"
+                raise testtools.TestCase.skipException(msg)
+            return f(self, *func_args, **func_kwargs)
+        return wrapper
+    return decorator
 
 
 def skip_without_vpc(*args, **kwargs):
@@ -219,6 +241,24 @@ def skip_without_vpc(*args, **kwargs):
     return decorator
 
 
+def check_network_feature_enabled(ext_name):
+    ext_list = CONF['network-feature-enabled']['api_extensions']
+    if 'all' not in ext_list and ext_name not in ext_list:
+        msg = ("Skipped network test as %s is not available" % ext_name)
+        raise testtools.TestCase.skipException(msg)
+
+
+def skip_without_network_feature(ext_name, *args, **kwargs):
+    """A decorator useful to skip tests without specified network extension."""
+    def decorator(f):
+        @functools.wraps(f)
+        def wrapper(self, *func_args, **func_kwargs):
+            check_network_feature_enabled(ext_name)
+            return f(self, *func_args, **func_kwargs)
+        return wrapper
+    return decorator
+
+
 class EC2TestCase(base.BaseTestCase):
     """Recommended to use as base class for boto related test."""
 
@@ -231,6 +271,9 @@ class EC2TestCase(base.BaseTestCase):
     @safe_setup
     def setUpClass(cls):
         super(EC2TestCase, cls).setUpClass()
+        if not CONF.service_available.ec2api:
+            raise cls.skipException("ec2api is disabled")
+
         cls.client = botocoreclient.get_ec2_client(
             CONF.aws.ec2_url, CONF.aws.aws_region,
             CONF.aws.aws_access, CONF.aws.aws_secret,
@@ -364,7 +407,10 @@ class EC2TestCase(base.BaseTestCase):
                           (cls.friendly_function_call_str(function, *pos_args,
                                                           **kw_args),
                            str((tb[0], tb[1], tb[2]))))
-                cls.cleanUpItem(function, pos_args, kw_args)
+                res = cls.cleanUpItem(function, pos_args, kw_args)
+                if not res:
+                    fail_count += 1
+                    LOG.error('Failure in cleanup for: %s' % str(kw_args))
             except BaseException:
                 fail_count += 1
                 LOG.exception('Failure in cleanup for: %s' % str(kw_args))
@@ -375,6 +421,7 @@ class EC2TestCase(base.BaseTestCase):
     @classmethod
     def cleanUpItem(cls, function, pos_args, kw_args):
         attempts_left = 10
+        interval = 1
         deleted = False
         while not deleted and attempts_left > 0:
             try:
@@ -390,11 +437,13 @@ class EC2TestCase(base.BaseTestCase):
                         waiter().wait_delete(obj_id)
                     except botocore.exceptions.ClientError as e:
                         LOG.exception('Exception occured in cleanup waiting')
+                        return False
             except botocore.exceptions.ClientError as e:
                 error_code = e.response['Error']['Code']
                 for err in cls._VALID_CLEANUP_ERRORS:
                     if err in error_code:
                         deleted = True
+                        break
                 else:
                     hook_res = False
                     key = (function.__name__, error_code)
@@ -405,10 +454,13 @@ class EC2TestCase(base.BaseTestCase):
                         hook_res = hook(obj_id)
                     if not hook_res:
                         LOG.error('Cleanup failed: %s', e, exc_info=True)
-                        return
+                        return False
                     LOG.error('Retrying cleanup due to: %s', e)
-                    time.sleep(1)
+                    time.sleep(interval)
                     attempts_left -= 1
+                    interval += 1
+
+        return deleted
 
     @classmethod
     def friendly_function_name_simple(cls, call_able):
@@ -477,6 +529,8 @@ class EC2TestCase(base.BaseTestCase):
                 data = cls.client.describe_addresses(
                     Filters=[{'Name': 'association-id', 'Values': [assoc_id]}])
 
+            LOG.debug('Addresses: %s' % str(data.get('Addresses')))
+
             if ('Addresses' in data and len(data['Addresses']) == 1 and
                     data['Addresses'][0].get('InstanceId')):
                 return 'available'
