@@ -4,15 +4,26 @@
 SERVICE_USERNAME=ec2api
 SERVICE_PASSWORD=ec2api
 SERVICE_TENANT=service
+# this domain name will be used for project and user
+SERVICE_DOMAIN_NAME=Default
+EC2API_PORT=8788
 CONNECTION="mysql://ec2api:ec2api@127.0.0.1/ec2api?charset=utf8"
 LOG_DIR=/var/log/ec2api
 CONF_DIR=/etc/ec2api
 NOVA_CONF=/etc/nova/nova.conf
 SIGNING_DIR=/var/cache/ec2api
+CONF_FILE=$CONF_DIR/ec2api.conf
+APIPASTE_FILE=$CONF_DIR/api-paste.ini
+
+AUTH_CACHE_DIR=${AUTH_CACHE_DIR:-/var/cache/ec2api}
 
 #Check for environment
-if [[ -z "$OS_AUTH_URL" || -z "$OS_USERNAME" || -z "$OS_PASSWORD" || -z "$OS_TENANT_NAME" ]]; then
-    echo "Please set OS_AUTH_URL, OS_USERNAME, OS_PASSWORD and OS_TENANT_NAME"
+if [[ -z "$OS_AUTH_URL" || -z "$OS_USERNAME" || -z "$OS_PASSWORD" ]]; then
+    echo "Please set OS_AUTH_URL, OS_USERNAME, OS_PASSWORD"
+    exit 1
+fi
+if [[ -z "$OS_TENANT_NAME" && -z "$OS_PROJECT_NAME" ]]; then
+    echo "Please set OS_TENANT_NAME or OS_PROJECT_NAME"
     exit 1
 fi
 
@@ -106,17 +117,18 @@ get_id () {
 get_user() {
     local username=$1
 
-    local user_id=$(get_data 2 $username 1 keystone user-list)
+    local user_id=$(openstack user show $username -f value -c id 2>/dev/null)
 
     if [ -n "$user_id" ]; then
         echo "Found existing $username user" >&2
         echo $user_id
     else
         echo "Creating $username user..." >&2
-        get_id keystone user-create --name=$username \
-                                    --pass="$SERVICE_PASSWORD" \
-                                    --tenant $SERVICE_TENANT \
-                                    --email=$username@example.com
+        openstack user create -f value -c id \
+                              $username \
+                              --password "$SERVICE_PASSWORD" \
+                              --project $SERVICE_TENANT \
+                              --email $username@example.com
     fi
 }
 
@@ -126,19 +138,19 @@ add_role() {
     local role_id=$3
     local username=$4
 
-    user_roles=$(keystone user-role-list \
-                          --user_id $user_id\
-                          --tenant $tenant 2>/dev/null)
+    user_roles=$(openstack user role list -f value -c ID \
+                                          $user_id \
+                                          --project $tenant 2>/dev/null)
     die_if_not_set $LINENO user_roles "Fail to get user_roles for tenant($tenant) and user_id($user_id)"
-    existing_role=$(get_data 1 $role_id 1 echo "$user_roles")
+    existing_role=$(echo "$user_roles" | grep $role_id || true)
     if [ -n "$existing_role" ]
     then
         echo "User $username already has role $role_id" >&2
         return
     fi
-    keystone user-role-add --tenant $tenant \
-             --user_id $user_id \
-             --role_id $role_id
+    openstack role add $role_id \
+                       --user $user_id \
+                       --project $tenant
 }
 
 
@@ -189,27 +201,28 @@ function iniget() {
 # Copy an option from Nova INI file or from environment if it's set
 function copynovaopt() {
     local option_name=$1
+    local option_group=$2
     local env_var
     local option
     env_var=${option_name^^}
     if [ ${!env_var+x} ]; then
         option=${!env_var}
-    elif ini_has_option "$NOVA_CONF" DEFAULT $option_name; then
-        option=$(iniget $NOVA_CONF DEFAULT $option_name)
+    elif ini_has_option "$NOVA_CONF" $option_group $option_name; then
+        option=$(iniget $NOVA_CONF $option_group $option_name)
     else
         return 0
     fi
-    iniset $CONF_FILE DEFAULT $option_name $option
+    iniset $CONF_FILE $option_group $option_name $option
 }
 
-if [[ -n $(keystone catalog --service network) ]]; then
+if [[ -n $(openstack catalog show network) ]]; then
     VPC_SUPPORT="True"
 else
     VPC_SUPPORT="False"
 fi
 if [[ "$VPC_SUPPORT" == "True" && -z "$EXTERNAL_NETWORK" ]]; then
     declare -a newtron_output
-    readarray -s 3 -t newtron_output < <(neutron net-external-list)
+    readarray -s 3 -t newtron_output < <(openstack network list --external)
     if ((${#newtron_output[@]} < 2)); then
         reason="No external network is declared in Neutron."
     elif ((${#newtron_output[@]} > 2)); then
@@ -221,10 +234,10 @@ if [[ "$VPC_SUPPORT" == "True" && -z "$EXTERNAL_NETWORK" ]]; then
 fi
 
 #create keystone user with admin privileges
-ADMIN_ROLE=$(get_data 2 admin 1 keystone role-list)
-die_if_not_set $LINENO ADMIN_ROLE "Fail to get ADMIN_ROLE by 'keystone role-list' "
-SERVICE_TENANT_ID=$(get_data 2 service 1 keystone tenant-list)
-die_if_not_set $LINENO SERVICE_TENANT_ID "Fail to get service tenant 'keystone tenant-list' "
+ADMIN_ROLE=$(openstack role show admin -c id -f value)
+die_if_not_set $LINENO ADMIN_ROLE "Fail to get ADMIN_ROLE by 'openstack role show' "
+SERVICE_TENANT_ID=$(openstack project show service -c id -f value)
+die_if_not_set $LINENO SERVICE_TENANT_ID "Fail to get service tenant 'openstack project show' "
 
 echo ADMIN_ROLE $ADMIN_ROLE
 echo SERVICE_TENANT $SERVICE_TENANT
@@ -238,61 +251,54 @@ add_role $SERVICE_USERID $SERVICE_TENANT $ADMIN_ROLE $SERVICE_USERNAME
 echo Creating log dir
 sudo install -d $LOG_DIR --owner=$USER
 
-CONF_FILE=$CONF_DIR/ec2api.conf
-APIPASTE_FILE=$CONF_DIR/api-paste.ini
 #copy conf files (do not override it)
 echo Creating configs
 sudo mkdir -p /etc/ec2api > /dev/null
 if [ ! -s $CONF_FILE ]; then
-    sudo cp etc/ec2api/ec2api.conf.sample $CONF_FILE
+    sudo touch $CONF_FILE
 fi
 if [ ! -s $APIPASTE_FILE ]; then
     sudo cp etc/ec2api/api-paste.ini $APIPASTE_FILE
 fi
 
-AUTH_HOST=${OS_AUTH_URL#*//}
-AUTH_HOST=${AUTH_HOST%:*}
-AUTH_CACHE_DIR=${AUTH_CACHE_DIR:-/var/cache/ec2api}
-AUTH_PORT=`keystone catalog|grep -A 9 identity|grep adminURL|awk '{print $4}'`
-AUTH_PORT=${AUTH_PORT##*:}
-AUTH_PORT=${AUTH_PORT%%/*}
-AUTH_PROTO=${OS_AUTH_URL%%:*}
-PUBLIC_URL=${OS_AUTH_URL%:*}:8788/
 
 #update default config with some values
+iniset $CONF_FILE DEFAULT ec2api_listen_port "$EC2API_PORT"
+iniset $CONF_FILE DEFAULT ec2_port "$EC2API_PORT"
 iniset $CONF_FILE DEFAULT api_paste_config $APIPASTE_FILE
 iniset $CONF_FILE DEFAULT logging_context_format_string "%(asctime)s.%(msecs)03d %(levelname)s %(name)s [%(request_id)s %(user_name)s %(project_name)s] %(instance)s%(message)s"
 iniset $CONF_FILE DEFAULT log_dir "$LOG_DIR"
 iniset $CONF_FILE DEFAULT verbose True
-iniset $CONF_FILE DEFAULT keystone_url "$OS_AUTH_URL"
+iniset $CONF_FILE DEFAULT keystone_ec2_tokens_url "$OS_AUTH_URL/v3/ec2tokens"
 iniset $CONF_FILE database connection "$CONNECTION"
 iniset $CONF_FILE DEFAULT full_vpc_support "$VPC_SUPPORT"
 iniset $CONF_FILE DEFAULT external_network "$EXTERNAL_NETWORK"
 
-iniset $CONF_FILE keystone_authtoken signing_dir $SIGNING_DIR
-iniset $CONF_FILE keystone_authtoken auth_host $AUTH_HOST
-iniset $CONF_FILE keystone_authtoken admin_user $SERVICE_USERNAME
-iniset $CONF_FILE keystone_authtoken admin_password $SERVICE_PASSWORD
-iniset $CONF_FILE keystone_authtoken admin_tenant_name $SERVICE_TENANT
-iniset $CONF_FILE keystone_authtoken auth_protocol $AUTH_PROTO
-iniset $CONF_FILE keystone_authtoken auth_port $AUTH_PORT
+GROUP_AUTHTOKEN="keystone_authtoken"
+iniset $CONF_FILE $GROUP_AUTHTOKEN signing_dir "$AUTH_CACHE_DIR"
+iniset $CONF_FILE $GROUP_AUTHTOKEN auth_uri "$OS_AUTH_URL"
+iniset $CONF_FILE $GROUP_AUTHTOKEN auth_url "$OS_AUTH_URL"
+iniset $CONF_FILE $GROUP_AUTHTOKEN username $SERVICE_USERNAME
+iniset $CONF_FILE $GROUP_AUTHTOKEN password $SERVICE_PASSWORD
+iniset $CONF_FILE $GROUP_AUTHTOKEN project_name $SERVICE_TENANT
+iniset $CONF_FILE $GROUP_AUTHTOKEN project_domain_name $SERVICE_DOMAIN_NAME
+iniset $CONF_FILE $GROUP_AUTHTOKEN user_domain_name $SERVICE_DOMAIN_NAME
+iniset $CONF_FILE $GROUP_AUTHTOKEN auth_type password
 
-iniset $CONF_FILE DEFAULT admin_user $SERVICE_USERNAME
-iniset $CONF_FILE DEFAULT admin_password $SERVICE_PASSWORD
-iniset $CONF_FILE DEFAULT admin_tenant_name $SERVICE_TENANT
 
 if [[ -f "$NOVA_CONF" ]]; then
     # NOTE(ft): use swift instead internal s3 server if enabled
-    if [[ -n $(keystone catalog --service object-store) ]] &&
-            [[ -n $(keystone catalog --service s3) ]]; then
-        copynovaopt s3_host
-        copynovaopt s3_port
-        copynovaopt s3_affix_tenant
-        copynovaopt s3_use_ssl
+    if [[ -n $(openstack catalog show object-store 2>/dev/null) ]] &&
+            [[ -n $(openstack catalog show s3 2>/dev/null) ]]; then
+        copynovaopt s3_host DEFAULT
+        copynovaopt s3_port DEFAULT
+        copynovaopt s3_affix_tenant DEFAULT
+        copynovaopt s3_use_ssl DEFAULT
     fi
-    copynovaopt cert_topic
-    copynovaopt rabbit_hosts
-    copynovaopt rabbit_password
+    copynovaopt cert_topic DEFAULT
+    copynovaopt rabbit_hosts oslo_messaging_rabbit
+    copynovaopt rabbit_password oslo_messaging_rabbit
+    copynovaopt rabbit_userid oslo_messaging_rabbit
     # TODO(ft): it's necessary to support other available messaging implementations
 
     nova_state_path=$(iniget $NOVA_CONF DEFAULT state_path)
